@@ -8,8 +8,9 @@ use dirs::home_dir;
 use indexmap::IndexMap;
 use std::collections::{HashMap, HashSet};
 use std::env;
+use std::fmt;
 use std::fs;
-use std::path::{PathBuf, MAIN_SEPARATOR};
+use std::path::PathBuf;
 use std::str::FromStr;
 
 pub fn compgen(
@@ -30,31 +31,26 @@ pub fn compgen(
             (last_arg.to_string(), None)
         }
     };
-    let redirect_symbols = shell.redirect_symbols();
-    let mut exist_redirect_symbol = false;
-    let new_args: Vec<String> = args
-        .iter()
-        .enumerate()
-        .map(|(i, v)| {
-            if !exist_redirect_symbol && redirect_symbols.contains(&v.as_str()) {
-                exist_redirect_symbol = true
-            }
-            if i == args.len() - 1 {
-                last.clone()
-            } else {
-                v.to_string()
-            }
-        })
-        .collect();
-    if exist_redirect_symbol {
-        return Ok("__argc_value=file".to_string());
-    }
     let cmd = Command::new(script_content)?;
+    let new_args: Vec<String> = if cmd.delegated() {
+        args.to_vec()
+    } else {
+        args.iter()
+            .enumerate()
+            .map(|(i, v)| {
+                if i == args.len() - 1 {
+                    last.clone()
+                } else {
+                    v.to_string()
+                }
+            })
+            .collect()
+    };
     let matcher = Matcher::new(&cmd, &new_args);
-    let compgen_values = matcher.compgen();
+    let compgen_values = matcher.compgen(shell);
     let mut default_nospace = unbalance.is_some();
     let mut prefix = unbalance.map(|v| v.to_string()).unwrap_or_default();
-    let mut candidates: IndexMap<String, (String, bool, CompKind)> = IndexMap::new();
+    let mut candidates: IndexMap<String, (String, bool, CompColor)> = IndexMap::new();
     let mut argc_fn = None;
     let mut argc_value = None;
     let mut argc_variables = vec![];
@@ -66,7 +62,7 @@ pub fn compgen(
             mod_quote(&mut last, &mut prefix, &mut default_nospace);
         }
     }
-    for (value, description, comp_kind) in compgen_values {
+    for (value, description, comp_color) in compgen_values {
         if value.starts_with("__argc_") {
             if let Some(fn_name) = value.strip_prefix("__argc_fn=") {
                 argc_fn = Some(fn_name.to_string());
@@ -88,16 +84,17 @@ pub fn compgen(
                 }
             }
         } else if value.starts_with(&last) && !multi_values.contains(&value) {
-            candidates.insert(value.clone(), (description, default_nospace, comp_kind));
+            candidates.insert(value.clone(), (description, default_nospace, comp_color));
         }
     }
     let mut argc_prefix = prefix.to_string();
     let mut argc_filter = last.to_string();
+    let mut argc_suffix = String::new();
     let mut argc_cd = None;
     if let Some(fn_name) = argc_fn {
         let output = if script_path.is_empty() {
-            let mut values = vec![];
             // complete for argc
+            let mut values = vec![];
             if fn_name == "_choice_completion" {
                 if new_args.len() == 3 {
                     values.extend(Shell::list().map(|v| v.name().to_string()))
@@ -106,7 +103,7 @@ pub fn compgen(
                 if new_args.len() == 3 {
                     values.extend(Shell::list().map(|v| v.name().to_string()))
                 } else {
-                    values.push("__argc_value=file".to_string());
+                    values.push("__argc_value=path".to_string());
                 }
             }
             Some(values.join("\n"))
@@ -114,8 +111,6 @@ pub fn compgen(
             let mut envs = HashMap::new();
             envs.insert("ARGC_COMPGEN".into(), "1".into());
             envs.insert("ARGC_OS".into(), env::consts::OS.to_string());
-            envs.insert("ARGC_PATH_SEP".into(), MAIN_SEPARATOR.into());
-            envs.insert("ARGC_DESCRIBE".into(), shell.with_description().to_string());
             envs.insert("ARGC_FILTER".into(), argc_filter.clone());
             envs.insert("ARGC_LAST_ARG".into(), last_arg.to_string());
             if let Some(cwd) = get_current_dir() {
@@ -133,6 +128,11 @@ pub fn compgen(
                         argc_value = Some(stripped_value.to_lowercase());
                     } else if let Some(stripped_value) = value.strip_prefix("__argc_prefix=") {
                         argc_prefix = format!("{prefix}{stripped_value}");
+                    } else if let Some(stripped_value) = value.strip_prefix("__argc_suffix=") {
+                        if nospace {
+                            default_nospace = true;
+                        }
+                        argc_suffix = stripped_value.to_string();
                     } else if let Some(stripped_value) = value.strip_prefix("__argc_filter=") {
                         argc_filter = stripped_value.to_string();
                         mod_quote(&mut argc_filter, &mut argc_prefix, &mut default_nospace);
@@ -142,22 +142,25 @@ pub fn compgen(
                     if shell.is_generic() {
                         argc_variables.push(value.to_string());
                     }
-                } else if value.starts_with(&argc_filter) && !multi_values.contains(&value) {
-                    match candidates.get_mut(&value) {
-                        Some((v1, v2, _)) => {
-                            if v1.is_empty() && !description.is_empty() {
-                                *v1 = description.to_string();
+                } else {
+                    let value = format!("{value}{argc_suffix}");
+                    if value.starts_with(&argc_filter) && !multi_values.contains(&value) {
+                        match candidates.get_mut(&value) {
+                            Some((v1, v2, _)) => {
+                                if v1.is_empty() && !description.is_empty() {
+                                    *v1 = description.to_string();
+                                }
+                                if !*v2 && nospace {
+                                    *v2 = true
+                                }
                             }
-                            if !*v2 && nospace {
-                                *v2 = true
+                            None => {
+                                candidates.insert(
+                                    value.to_string(),
+                                    (description.to_string(), nospace, comp_type),
+                                );
                             }
                         }
-                        None => {
-                            candidates.insert(
-                                value.to_string(),
-                                (description.to_string(), nospace, comp_type),
-                            );
-                        }
                     }
                 }
             }
@@ -165,9 +168,9 @@ pub fn compgen(
     }
 
     if !argc_variables.is_empty() {
-        let mut prepend_candicates: IndexMap<String, (String, bool, CompKind)> = argc_variables
+        let mut prepend_candicates: IndexMap<String, (String, bool, CompColor)> = argc_variables
             .into_iter()
-            .map(|value| (value, (String::new(), false, CompKind::Value)))
+            .map(|value| (value, (String::new(), false, CompColor::of_value())))
             .collect();
         prepend_candicates.extend(candidates);
         candidates = prepend_candicates;
@@ -175,17 +178,26 @@ pub fn compgen(
 
     let mut candidates: Vec<CandidateValue> = candidates
         .into_iter()
-        .map(|(value, (description, nospace, comp_kind))| (value, description, nospace, comp_kind))
+        .map(|(value, (description, nospace, comp_color))| {
+            (value, description, nospace, comp_color)
+        })
         .collect();
 
     if !shell.is_generic() {
-        if let Some(argc_value) = argc_value.and_then(|v| convert_arg_value(&v)) {
+        if let Some(path_value) = argc_value.and_then(|v| convert_arg_value(&v)) {
             if let Some((value_prefix, value_filter, more_candidates)) =
-                shell.comp_argc_value(&argc_value, &argc_filter, &argc_cd, default_nospace)
+                path_value.compgen(shell, &argc_filter, &argc_suffix, &argc_cd, default_nospace)
             {
-                argc_prefix = format!("{argc_prefix}{value_prefix}");
-                argc_filter = value_filter;
-                candidates.extend(more_candidates)
+                if candidates.is_empty() || value_prefix.is_empty() {
+                    argc_prefix = format!("{argc_prefix}{value_prefix}");
+                    argc_filter = value_filter;
+                    candidates.extend(more_candidates)
+                } else {
+                    candidates.extend(more_candidates.into_iter().map(|mut v| {
+                        v.0 = format!("{value_prefix}{}", v.0);
+                        v
+                    }))
+                }
             }
         }
     }
@@ -222,62 +234,6 @@ pub fn compgen(
     Ok(values.join("\n"))
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
-pub(crate) enum CompKind {
-    Flag,
-    Option,
-    Command,
-    Dir,
-    File,
-    FileExe,
-    Symlink,
-    ValueOther,
-    ValueAnother,
-    ValueEmphasis,
-    ValueSubtle,
-    Value,
-}
-
-impl FromStr for CompKind {
-    type Err = anyhow::Error;
-
-    fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
-        match s.to_ascii_lowercase().as_str() {
-            "flag" => Ok(Self::Flag),
-            "option" => Ok(Self::Option),
-            "command" => Ok(Self::Command),
-            "dir" => Ok(Self::Dir),
-            "file" => Ok(Self::File),
-            "fileexe" => Ok(Self::FileExe),
-            "symlink" => Ok(Self::Symlink),
-            "valueother" => Ok(Self::ValueOther),
-            "valueanother" => Ok(Self::ValueAnother),
-            "valueemphasis" => Ok(Self::ValueEmphasis),
-            "valuesubtle" => Ok(Self::ValueSubtle),
-            _ => Ok(Self::Value),
-        }
-    }
-}
-
-impl CompKind {
-    pub(crate) fn name(&self) -> &str {
-        match self {
-            Self::Flag => "flag",
-            Self::Option => "option",
-            Self::Command => "command",
-            Self::Dir => "dir",
-            Self::File => "file",
-            Self::FileExe => "exe",
-            Self::Symlink => "symlink",
-            Self::ValueOther => "valueOther",
-            Self::ValueAnother => "valueAnother",
-            Self::ValueEmphasis => "valueEmphasis",
-            Self::ValueSubtle => "valueSubtle",
-            Self::Value => "value",
-        }
-    }
-}
-
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum Shell {
     Bash,
@@ -311,7 +267,7 @@ impl FromStr for Shell {
     }
 }
 
-pub(crate) type CandidateValue = (String, String, bool, CompKind); // value, description, nospace
+pub(crate) type CandidateValue = (String, String, bool, CompColor); // value, description, nospace, comp_color
 
 impl Shell {
     pub fn list() -> [Shell; 7] {
@@ -386,8 +342,8 @@ impl Shell {
                 candidates
                     .into_iter()
                     .enumerate()
-                    .map(|(i, (value, description, nospace, _comp_kind))| {
-                        let mut new_value = self.escape(&value);
+                    .map(|(i, (value, description, nospace, _comp_color))| {
+                        let mut new_value = value;
                         if i == 0 && add_space_to_first_candidate {
                             new_value = format!(" {}", new_value)
                         };
@@ -403,18 +359,18 @@ impl Shell {
             }
             Shell::Elvish | Shell::Powershell => candidates
                 .into_iter()
-                .map(|(value, description, nospace, comp_kind)| {
+                .map(|(value, description, nospace, comp_color)| {
                     let new_value = self.combine_value(prefix, &value);
                     let display = if value.is_empty() { " ".into() } else { value };
                     let description = self.comp_description(&description, "", "");
                     let space: &str = if nospace { "0" } else { "1" };
-                    let color = self.color(comp_kind, no_color);
+                    let color = self.color(comp_color, no_color);
                     format!("{new_value}\t{space}\t{display}\t{description}\t{color}")
                 })
                 .collect::<Vec<String>>(),
             Shell::Fish => candidates
                 .into_iter()
-                .map(|(value, description, _nospace, _comp_kind)| {
+                .map(|(value, description, _nospace, _comp_color)| {
                     let new_value = self.combine_value(prefix, &value);
                     let description = self.comp_description(&description, "\t", "");
                     format!("{new_value}{description}")
@@ -422,11 +378,11 @@ impl Shell {
                 .collect::<Vec<String>>(),
             Shell::Generic => candidates
                 .into_iter()
-                .map(|(value, description, nospace, comp_kind)| {
-                    let comp_kind = format!("\t/kind:{}", comp_kind.name());
+                .map(|(value, description, nospace, comp_color)| {
+                    let comp_color = format!("\t/color:{}", comp_color.ser());
                     let description = self.comp_description(&description, "\t", "");
                     let space: &str = if nospace { "\0" } else { "" };
-                    format!("{value}{space}{comp_kind}{description}")
+                    format!("{value}{space}{comp_color}{description}")
                 })
                 .collect::<Vec<String>>(),
             Shell::Nushell => candidates
@@ -465,7 +421,7 @@ impl Shell {
                 .collect::<Vec<String>>(),
             Shell::Zsh => candidates
                 .into_iter()
-                .map(|(value, description, nospace, comp_kind)| {
+                .map(|(value, description, nospace, comp_color)| {
                     let prefix = if let Some((_, i)) = unbalance_quote(prefix) {
                         prefix
                             .char_indices()
@@ -480,7 +436,7 @@ impl Shell {
                         escape_chars(&value, self.need_escape_chars(), "\\").replace("\\:", ":");
                     let display = value.replace(':', "\\:");
                     let description = self.comp_description(&description, ":", "");
-                    let color = self.color(comp_kind, no_color);
+                    let color = self.color(comp_color, no_color);
                     let space = if nospace { "" } else { " " };
                     format!("{new_value}{space}\t{display}{description}\t{match_value}\t{color}")
                 })
@@ -516,51 +472,25 @@ impl Shell {
         }
     }
 
-    fn color(&self, comp_kind: CompKind, no_color: bool) -> &'static str {
+    pub(crate) fn color(&self, comp_color: CompColor, no_color: bool) -> String {
         match self {
             Shell::Elvish => {
                 if no_color {
-                    return "default";
-                }
-                match comp_kind {
-                    CompKind::Flag => "cyan",
-                    CompKind::Option => "cyan bold",
-                    CompKind::Command => "magenta",
-                    CompKind::Dir => "blue bold",
-                    CompKind::File => "default",
-                    CompKind::FileExe => "green bold",
-                    CompKind::Symlink => "cyan bold",
-                    CompKind::ValueOther => "yellow",
-                    CompKind::ValueAnother => "red",
-                    CompKind::ValueEmphasis => "green bold",
-                    CompKind::ValueSubtle => "green dim",
-                    CompKind::Value => "green",
+                    return "default".into();
                 }
+                comp_color.style()
             }
             Shell::Powershell | Shell::Zsh => {
                 if no_color {
-                    return "39";
-                }
-                match comp_kind {
-                    CompKind::Flag => "36",
-                    CompKind::Option => "1;36",
-                    CompKind::Command => "35",
-                    CompKind::Dir => "1;34",
-                    CompKind::File => "39",
-                    CompKind::FileExe => "1;32",
-                    CompKind::Symlink => "1;36",
-                    CompKind::ValueOther => "33",
-                    CompKind::ValueAnother => "31",
-                    CompKind::ValueEmphasis => "1;32",
-                    CompKind::ValueSubtle => "2;32",
-                    CompKind::Value => "32",
+                    return "39".into();
                 }
+                comp_color.ansi_code()
             }
-            _ => "",
+            _ => String::new(),
         }
     }
 
-    fn redirect_symbols(&self) -> Vec<&str> {
+    pub(crate) fn redirect_symbols(&self) -> Vec<&str> {
         match self {
             Shell::Nushell => vec![],
             _ => vec!["<", ">"],
@@ -605,37 +535,236 @@ impl Shell {
         self.is_windows_mode() && self == &Self::Powershell
     }
 
-    fn comp_argc_value(
+    fn combine_value(&self, prefix: &str, value: &str) -> String {
+        let mut new_value = format!("{prefix}{value}");
+        if unbalance_quote(prefix).is_none() {
+            new_value = self.escape(&new_value);
+        }
+        new_value
+    }
+
+    fn comp_description(&self, description: &str, prefix: &str, suffix: &str) -> String {
+        if description.is_empty() || !self.with_description() {
+            String::new()
+        } else {
+            format!("{prefix}{}{suffix}", truncate_description(description))
+        }
+    }
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub(crate) struct CompColor {
+    pub(crate) code: ColorCode,
+    pub(crate) style: ColorStyle,
+}
+
+impl CompColor {
+    pub(crate) fn new(code: ColorCode, style: ColorStyle) -> Self {
+        Self { code, style }
+    }
+
+    pub(crate) fn deser(s: &str) -> Result<Self> {
+        if let Some((code, style)) = s.split_once(',') {
+            if let (Ok(code), Ok(style)) = (code.parse(), style.parse()) {
+                return Ok(Self::new(code, style));
+            }
+        } else if let Ok(code) = s.parse() {
+            return Ok(Self::new(code, ColorStyle::Regular));
+        }
+        bail!("Invalid CompColor value")
+    }
+
+    pub(crate) fn ser(&self) -> String {
+        if self.style == ColorStyle::Regular {
+            format!("{}", self.code)
+        } else {
+            format!("{},{}", self.code, self.style)
+        }
+    }
+
+    pub(crate) fn ansi_code(&self) -> String {
+        let mut ret = if self.style != ColorStyle::Regular {
+            format!("{};", self.style.ansi_code())
+        } else {
+            String::new()
+        };
+        ret.push_str(self.code.ansi_code());
+        ret
+    }
+
+    pub(crate) fn style(&self) -> String {
+        let mut ret = self.code.to_string();
+        if self.style != ColorStyle::Regular {
+            ret.push_str(&format!(" {}", self.style))
+        }
+        ret
+    }
+
+    pub(crate) fn of_flag() -> Self {
+        Self::new(ColorCode::Cyan, ColorStyle::Regular)
+    }
+
+    pub(crate) fn of_option() -> Self {
+        Self::new(ColorCode::Cyan, ColorStyle::Bold)
+    }
+
+    pub(crate) fn of_command() -> Self {
+        Self::new(ColorCode::Magenta, ColorStyle::Regular)
+    }
+
+    pub(crate) fn of_dir() -> Self {
+        Self::new(ColorCode::Blue, ColorStyle::Bold)
+    }
+
+    pub(crate) fn of_file() -> Self {
+        Self::new(ColorCode::Default, ColorStyle::Regular)
+    }
+
+    pub(crate) fn of_file_exe() -> Self {
+        Self::new(ColorCode::Green, ColorStyle::Bold)
+    }
+
+    pub(crate) fn of_symlink() -> Self {
+        Self::new(ColorCode::Cyan, ColorStyle::Bold)
+    }
+
+    pub(crate) fn of_value() -> Self {
+        Self::new(ColorCode::Green, ColorStyle::Regular)
+    }
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub(crate) enum ColorCode {
+    Black,
+    Red,
+    Green,
+    Yellow,
+    Blue,
+    Magenta,
+    Cyan,
+    White,
+    Default,
+}
+
+impl FromStr for ColorCode {
+    type Err = anyhow::Error;
+
+    fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
+        match s {
+            "black" => Ok(Self::Black),
+            "red" => Ok(Self::Red),
+            "green" => Ok(Self::Green),
+            "yellow" => Ok(Self::Yellow),
+            "blue" => Ok(Self::Blue),
+            "magenta" => Ok(Self::Magenta),
+            "cyan" => Ok(Self::Cyan),
+            "white" => Ok(Self::White),
+            "default" => Ok(Self::Default),
+            _ => bail!("Invalid ColorCode value"),
+        }
+    }
+}
+
+impl fmt::Display for ColorCode {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::Black => write!(f, "black"),
+            Self::Red => write!(f, "red"),
+            Self::Green => write!(f, "green"),
+            Self::Yellow => write!(f, "yellow"),
+            Self::Blue => write!(f, "blue"),
+            Self::Magenta => write!(f, "magenta"),
+            Self::Cyan => write!(f, "cyan"),
+            Self::White => write!(f, "white"),
+            Self::Default => write!(f, "default"),
+        }
+    }
+}
+
+impl ColorCode {
+    pub fn ansi_code(&self) -> &str {
+        match self {
+            Self::Black => "30",
+            Self::Red => "31",
+            Self::Green => "32",
+            Self::Yellow => "33",
+            Self::Blue => "34",
+            Self::Magenta => "35",
+            Self::Cyan => "36",
+            Self::White => "37",
+            Self::Default => "39",
+        }
+    }
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub(crate) enum ColorStyle {
+    Regular,
+    Bold,
+}
+
+impl FromStr for ColorStyle {
+    type Err = anyhow::Error;
+
+    fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
+        match s {
+            "regular" => Ok(Self::Regular),
+            "bold" => Ok(Self::Bold),
+            _ => bail!("Invalid CodeStyle value"),
+        }
+    }
+}
+
+impl fmt::Display for ColorStyle {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::Regular => write!(f, "regular"),
+            Self::Bold => write!(f, "bold"),
+        }
+    }
+}
+
+impl ColorStyle {
+    pub fn ansi_code(&self) -> &str {
+        match self {
+            Self::Regular => "0",
+            Self::Bold => "1",
+        }
+    }
+}
+
+pub(crate) struct ArgcPathValue {
+    pub(crate) is_dir: bool,
+    pub(crate) exts: Vec<String>,
+}
+
+impl ArgcPathValue {
+    fn compgen(
         &self,
-        argc_value: &str,
+        shell: Shell,
         value: &str,
+        suffix: &str,
         cd: &Option<String>,
         default_nospace: bool,
     ) -> Option<(String, String, Vec<CandidateValue>)> {
-        let (dir_only, exts) = if argc_value == "__argc_value=dir" {
-            (true, None)
-        } else if argc_value == "__argc_value=file" {
-            (false, None)
-        } else if let Some(stripped_value) = argc_value.strip_prefix("__argc_value=file:") {
-            let exts: Vec<String> = stripped_value.split(',').map(|v| v.to_string()).collect();
-            (false, Some(exts))
-        } else {
-            return None;
-        };
-        let (search_dir, filter, prefix) = self.resolve_path(value, cd)?;
+        let (search_dir, filter, prefix) = self.resolve_path(shell, value, cd)?;
         if !search_dir.is_dir() {
             return None;
         }
 
-        let is_windows_mode = self.is_windows_mode();
-        let exts = exts.unwrap_or_default();
+        let is_windows_mode = shell.is_windows_mode();
 
         let (value_prefix, path_sep) = if is_windows_mode {
             if !value.contains(&prefix) {
                 return Some((
                     "".into(),
                     "".into(),
-                    vec![(format!("{prefix}{filter}"), "".into(), true, CompKind::Dir)],
+                    vec![(
+                        format!("{prefix}{filter}"),
+                        "".into(),
+                        true,
+                        CompColor::of_dir(),
+                    )],
                 ));
             }
             let value_prefix = if prefix.is_empty() { ".\\" } else { "" };
@@ -645,7 +774,7 @@ impl Shell {
                 return Some((
                     "".into(),
                     "".into(),
-                    vec![("~/".into(), "".into(), true, CompKind::Dir)],
+                    vec![("~/".into(), "".into(), true, CompColor::of_dir())],
                 ));
             }
             ("", "/")
@@ -687,25 +816,28 @@ impl Shell {
                 };
             }
             let is_dir = meta.is_dir();
-            if !is_dir && dir_only {
+            if !is_dir && self.is_dir {
                 continue;
             }
             if !is_dir
-                && !exts.is_empty()
-                && exts.iter().all(|v| !file_name.to_lowercase().ends_with(v))
+                && !self.exts.is_empty()
+                && self
+                    .exts
+                    .iter()
+                    .all(|v| !file_name.to_lowercase().ends_with(v))
             {
                 continue;
             }
             let path_value = if is_dir {
                 format!("{value_prefix}{file_name}{}", path_sep)
             } else {
-                format!("{value_prefix}{file_name}")
+                format!("{value_prefix}{file_name}{suffix}")
             };
 
-            let comp_kind = if is_dir {
-                CompKind::Dir
+            let comp_color = if is_dir {
+                CompColor::of_dir()
             } else if is_symlink {
-                CompKind::Symlink
+                CompColor::of_symlink()
             } else {
                 #[cfg(not(windows))]
                 let is_executable = {
@@ -718,22 +850,27 @@ impl Shell {
                     path_exts.iter().any(|v| new_file_name.ends_with(v))
                 };
                 if is_executable {
-                    CompKind::FileExe
+                    CompColor::of_file_exe()
                 } else {
-                    CompKind::File
+                    CompColor::of_file()
                 }
             };
             let nospace = if default_nospace { true } else { is_dir };
-            output.push((path_value, String::new(), nospace, comp_kind))
+            output.push((path_value, String::new(), nospace, comp_color))
         }
 
-        output.sort_by(|a, b| a.0.cmp(&b.0));
+        output.sort_by(|a, b| natord::compare_ignore_case(&a.0, &b.0));
 
         Some((prefix, filter, output))
     }
 
-    fn resolve_path(&self, value: &str, cd: &Option<String>) -> Option<(PathBuf, String, String)> {
-        let is_windows_mode = self.is_windows_mode();
+    fn resolve_path(
+        &self,
+        shell: Shell,
+        value: &str,
+        cd: &Option<String>,
+    ) -> Option<(PathBuf, String, String)> {
+        let is_windows_mode = shell.is_windows_mode();
         let (value, sep, home_p, cwd_p) = if is_windows_mode {
             (value.replace('/', "\\"), "\\", "~\\", ".\\")
         } else {
@@ -747,7 +884,7 @@ impl Shell {
             } else {
                 format!("{home_s}{}", &value[1..])
             };
-            let (trims, prefix) = if self.need_expand_tilde() {
+            let (trims, prefix) = if shell.need_expand_tilde() {
                 (0, "")
             } else {
                 (home_s.len(), "~")
@@ -793,21 +930,32 @@ impl Shell {
         let prefix = format!("{prefix}{}", &path[trims..]);
         Some((PathBuf::from(path), filter, prefix))
     }
+}
 
-    fn combine_value(&self, prefix: &str, value: &str) -> String {
-        let mut new_value = format!("{prefix}{value}");
-        if unbalance_quote(prefix).is_none() {
-            new_value = self.escape(&new_value);
-        }
-        new_value
-    }
-
-    fn comp_description(&self, description: &str, prefix: &str, suffix: &str) -> String {
-        if description.is_empty() || !self.with_description() {
-            String::new()
-        } else {
-            format!("{prefix}{}{suffix}", truncate_description(description))
-        }
+fn convert_arg_value(value: &str) -> Option<ArgcPathValue> {
+    if let Some(stripped_value) = value
+        .strip_prefix("path:")
+        .or_else(|| value.strip_prefix("file:"))
+    {
+        Some(ArgcPathValue {
+            is_dir: false,
+            exts: stripped_value.split(',').map(|v| v.to_string()).collect(),
+        })
+    } else if ["path", "file", "arg", "any"]
+        .iter()
+        .any(|v| value.contains(v))
+    {
+        Some(ArgcPathValue {
+            is_dir: false,
+            exts: vec![],
+        })
+    } else if value.contains("dir") || value.contains("folder") {
+        Some(ArgcPathValue {
+            is_dir: true,
+            exts: vec![],
+        })
+    } else {
+        None
     }
 }
 
@@ -817,10 +965,13 @@ fn parse_candidate_value(input: &str) -> CandidateValue {
     let mut value = String::new();
     let mut description = String::new();
     let mut nospace = false;
-    let mut comp_kind = CompKind::Value;
+    let mut comp_color = CompColor::of_value();
     if parts_len >= 2 {
-        if let Some(kind_value) = parts[1].strip_prefix("/kind:").and_then(|v| v.parse().ok()) {
-            comp_kind = kind_value;
+        if let Some(color) = parts[1]
+            .strip_prefix("/color:")
+            .and_then(|v| CompColor::deser(v).ok())
+        {
+            comp_color = color;
             description = parts[2..].join("\t");
         } else {
             description = parts[1..].join("\t");
@@ -828,37 +979,29 @@ fn parse_candidate_value(input: &str) -> CandidateValue {
     }
     if parts_len > 0 {
         if let Some(stripped_value) = parts.first().and_then(|v| v.strip_suffix('\0')) {
-            value = stripped_value.to_string();
+            value = stripped_value.trim().to_string();
             nospace = true;
         } else {
-            value = parts[0].to_string();
+            value = parts[0].trim().to_string();
         }
     }
-    (value, description, nospace, comp_kind)
-}
-
-fn convert_arg_value(value: &str) -> Option<String> {
-    if value.starts_with("file:") {
-        Some(format!("__argc_value={value}"))
-    } else if ["path", "file", "arg", "any"]
-        .iter()
-        .any(|v| value.contains(v))
-    {
-        Some("__argc_value=file".to_string())
-    } else if value.contains("dir") || value.contains("folder") {
-        Some("__argc_value=dir".to_string())
-    } else {
-        None
-    }
+    (value, description, nospace, comp_color)
 }
 
 fn truncate_description(description: &str) -> String {
     let max_width = 80;
-    let description = description.trim().replace('\t', "");
-    if description.len() < max_width {
+    let mut description = description.trim().replace('\t', "");
+    if description.starts_with('(') && description.ends_with(')') {
+        description = description
+            .trim_start_matches('(')
+            .trim_end_matches(')')
+            .to_string();
+    }
+    if description.chars().count() < max_width {
         description
     } else {
-        format!("{}...", &description[0..max_width - 3])
+        let truncated: String = description.chars().take(max_width).collect();
+        format!("{}...", truncated)
     }
 }
 
@@ -869,7 +1012,7 @@ fn unbalance_quote(value: &str) -> Option<(char, usize)> {
             return Some((ch, 0));
         }
     } else if value.ends_with(is_quote) {
-        let ch = value.chars().rev().next()?;
+        let ch = value.chars().next_back()?;
         if value.chars().filter(|c| *c == ch).count() % 2 == 1 {
             return Some((ch, value.len() - 1));
         }
@@ -950,44 +1093,62 @@ mod tests {
     use super::*;
 
     macro_rules! assert_parse_candidate_value {
-        ($input:expr, $value:expr, $desc:expr, $nospace:expr, $comp_kind:expr) => {
+        ($input:expr, $value:expr, $desc:expr, $nospace:expr, $comp_color:expr) => {
             assert_eq!(
                 parse_candidate_value($input),
-                ($value.to_string(), $desc.to_string(), $nospace, $comp_kind)
+                ($value.to_string(), $desc.to_string(), $nospace, $comp_color)
             )
         };
     }
 
     #[test]
     fn test_parse_candidate_value() {
-        assert_parse_candidate_value!("abc", "abc", "", false, CompKind::Value);
-        assert_parse_candidate_value!("abc\0", "abc", "", true, CompKind::Value);
-        assert_parse_candidate_value!("abc\tA value", "abc", "A value", false, CompKind::Value);
-        assert_parse_candidate_value!("abc\0\tA value", "abc", "A value", true, CompKind::Value);
+        assert_parse_candidate_value!("abc", "abc", "", false, CompColor::of_value());
+        assert_parse_candidate_value!("abc\0", "abc", "", true, CompColor::of_value());
+        assert_parse_candidate_value!(
+            "abc\tA value",
+            "abc",
+            "A value",
+            false,
+            CompColor::of_value()
+        );
+        assert_parse_candidate_value!(
+            "abc\0\tA value",
+            "abc",
+            "A value",
+            true,
+            CompColor::of_value()
+        );
         assert_parse_candidate_value!(
             "abc\0\tA value\tmore desc",
             "abc",
             "A value\tmore desc",
             true,
-            CompKind::Value
+            CompColor::of_value()
         );
         assert_parse_candidate_value!(
-            "abc\0\t/kind:command\tA value\tmore desc",
+            "abc\0\t/color:magenta\tA value\tmore desc",
             "abc",
             "A value\tmore desc",
             true,
-            CompKind::Command
+            CompColor::of_command()
         );
         assert_parse_candidate_value!(
-            "abc\0\t/kind:command\tA value",
+            "abc\0\t/color:cyan,bold\tA value",
             "abc",
             "A value",
             true,
-            CompKind::Command
+            CompColor::of_option()
         );
-        assert_parse_candidate_value!("abc\0\t/kind:command", "abc", "", true, CompKind::Command);
-        assert_parse_candidate_value!("abc\t/kind:command", "abc", "", false, CompKind::Command);
-        assert_parse_candidate_value!("", "", "", false, CompKind::Value);
+        assert_parse_candidate_value!("abc\0\t/color:cyan", "abc", "", true, CompColor::of_flag());
+        assert_parse_candidate_value!(
+            "abc\t/color:default",
+            "abc",
+            "",
+            false,
+            CompColor::of_file()
+        );
+        assert_parse_candidate_value!("", "", "", false, CompColor::of_value());
     }
 
     #[test]
