@@ -6,10 +6,10 @@ use std::{
 
 use crate::{
     command::Command,
-    compgen::CompKind,
+    compgen::CompColor,
     param::{FlagOptionParam, PositionalParam},
     utils::run_param_fns,
-    ArgcValue,
+    ArgcValue, Shell,
 };
 
 use either::Either;
@@ -25,7 +25,6 @@ pub(crate) struct Matcher<'a, 'b> {
     dashes: Vec<usize>,
     arg_comp: ArgComp,
     choices_fns: HashSet<&'a str>,
-    choices_values: HashMap<&'a str, Vec<String>>,
     script_path: Option<String>,
     term_width: Option<usize>,
     is_last_arg_option_assign: bool,
@@ -70,24 +69,107 @@ impl<'a, 'b> Matcher<'a, 'b> {
         let mut flag_option_args = vec![vec![]];
         let mut positional_args = vec![];
         let mut dashes = vec![];
-        let mut is_rest_args_positional = false;
         let mut is_last_arg_option_assign = false;
         let mut arg_comp = ArgComp::Any;
         let mut choices_fns = HashSet::new();
         let args_len = args.len();
-        if let Some(arg) = args.last() {
-            if arg.starts_with('-') {
-                arg_comp = ArgComp::FlagOrOption;
-            } else if !arg.is_empty() {
-                arg_comp = ArgComp::CommandOrPositional;
+        if root.delegated() {
+            positional_args = args.iter().skip(1).map(|v| v.as_str()).collect();
+        } else {
+            let mut is_rest_args_positional = false;
+            if let Some(arg) = args.last() {
+                if arg.starts_with('-') {
+                    arg_comp = ArgComp::FlagOrOption;
+                } else if !arg.is_empty() {
+                    arg_comp = ArgComp::CommandOrPositional;
+                }
             }
-        }
-        while arg_index < args_len {
-            let cmd = cmds[cmd_level].1;
-            let arg = args[arg_index].as_str();
-            if arg == "--" {
-                dashes.push(positional_args.len());
-                if is_rest_args_positional {
+            while arg_index < args_len {
+                let cmd = cmds[cmd_level].1;
+                let arg = args[arg_index].as_str();
+                if arg == "--" {
+                    dashes.push(positional_args.len());
+                    if is_rest_args_positional {
+                        add_positional_arg(
+                            &mut positional_args,
+                            arg,
+                            &mut is_rest_args_positional,
+                            cmd,
+                        );
+                    }
+                } else if is_rest_args_positional
+                    || !dashes.is_empty()
+                    || (cmd.no_flags_options_subcommands() && !KNOWN_OPTIONS.contains(&arg))
+                {
+                    add_positional_arg(
+                        &mut positional_args,
+                        arg,
+                        &mut is_rest_args_positional,
+                        cmd,
+                    );
+                } else if arg.starts_with('-') {
+                    if let Some((k, v)) = arg.split_once('=') {
+                        let param = cmd.find_flag_option(k);
+                        if arg_index == args_len - 1 {
+                            if let Some(param) = param {
+                                arg_comp = ArgComp::OptionValue(param.name.clone(), 0)
+                            }
+                            is_last_arg_option_assign = true;
+                        }
+                        if let Some((choices_fn, validate)) =
+                            param.and_then(|v| v.choices_fn.as_ref())
+                        {
+                            if *validate {
+                                choices_fns.insert(choices_fn.as_str());
+                            }
+                        }
+                        flag_option_args[cmd_level].push((
+                            k,
+                            vec![v],
+                            param.map(|v| v.name.as_str()),
+                        ));
+                    } else if let Some(param) = cmd.find_flag_option(arg) {
+                        if let Some((choices_fn, validate)) = param.choices_fn.as_ref() {
+                            if *validate {
+                                choices_fns.insert(choices_fn.as_str());
+                            }
+                        }
+                        match_flag_option(
+                            &mut flag_option_args[cmd_level],
+                            args,
+                            &mut arg_index,
+                            param,
+                            &mut arg_comp,
+                        );
+                    } else if let Some(mut list) = match_combine_shorts(cmd, arg) {
+                        let name = list.pop().and_then(|v| v.2).unwrap();
+                        let param = cmd.find_flag_option(name).unwrap();
+                        if let Some((choices_fn, validate)) = param.choices_fn.as_ref() {
+                            if *validate {
+                                choices_fns.insert(choices_fn.as_str());
+                            }
+                        }
+                        flag_option_args[cmd_level].extend(list);
+                        match_flag_option(
+                            &mut flag_option_args[cmd_level],
+                            args,
+                            &mut arg_index,
+                            param,
+                            &mut arg_comp,
+                        );
+                    } else {
+                        flag_option_args[cmd_level].push((arg, vec![], None));
+                    }
+                } else if let Some(subcmd) = cmd.find_subcommand(arg) {
+                    cmd_level += 1;
+                    cmds.push((
+                        arg,
+                        subcmd,
+                        subcmd.name.clone().unwrap_or_else(|| arg.to_string()),
+                        arg_index,
+                    ));
+                    flag_option_args.push(vec![]);
+                } else {
                     add_positional_arg(
                         &mut positional_args,
                         arg,
@@ -95,87 +177,23 @@ impl<'a, 'b> Matcher<'a, 'b> {
                         cmd,
                     );
                 }
-            } else if is_rest_args_positional
-                || !dashes.is_empty()
-                || (cmd.no_flags_options_subcommands() && !KNOWN_OPTIONS.contains(&arg))
-            {
-                add_positional_arg(&mut positional_args, arg, &mut is_rest_args_positional, cmd);
-            } else if arg.starts_with('-') {
-                if let Some((k, v)) = arg.split_once('=') {
-                    let param = cmd.find_flag_option(k);
-                    if arg_index == args_len - 1 {
-                        if let Some(param) = param {
-                            arg_comp = ArgComp::OptionValue(param.name.clone(), 0)
-                        }
-                        is_last_arg_option_assign = true;
-                    }
-                    if let Some((choices_fn, validate)) = param.and_then(|v| v.choices_fn.as_ref())
-                    {
-                        if *validate {
-                            choices_fns.insert(choices_fn.as_str());
-                        }
-                    }
-                    flag_option_args[cmd_level].push((k, vec![v], param.map(|v| v.name.as_str())));
-                } else if let Some(param) = cmd.find_flag_option(arg) {
-                    if let Some((choices_fn, validate)) = param.choices_fn.as_ref() {
-                        if *validate {
-                            choices_fns.insert(choices_fn.as_str());
-                        }
-                    }
-                    match_flag_option(
-                        &mut flag_option_args[cmd_level],
-                        args,
-                        &mut arg_index,
-                        param,
-                        &mut arg_comp,
-                    );
-                } else if let Some(mut list) = match_combine_shorts(cmd, arg) {
-                    let name = list.pop().and_then(|v| v.2).unwrap();
-                    let param = cmd.find_flag_option(name).unwrap();
-                    if let Some((choices_fn, validate)) = param.choices_fn.as_ref() {
-                        if *validate {
-                            choices_fns.insert(choices_fn.as_str());
-                        }
-                    }
-                    flag_option_args[cmd_level].extend(list);
-                    match_flag_option(
-                        &mut flag_option_args[cmd_level],
-                        args,
-                        &mut arg_index,
-                        param,
-                        &mut arg_comp,
-                    );
-                } else {
-                    flag_option_args[cmd_level].push((arg, vec![], None));
-                }
-            } else if let Some(subcmd) = cmd.find_subcommand(arg) {
-                cmd_level += 1;
-                cmds.push((
-                    arg,
-                    subcmd,
-                    subcmd.name.clone().unwrap_or_else(|| arg.to_string()),
-                    arg_index,
-                ));
-                flag_option_args.push(vec![]);
-            } else {
-                add_positional_arg(&mut positional_args, arg, &mut is_rest_args_positional, cmd);
+                arg_index += 1;
             }
-            arg_index += 1;
-        }
 
-        if is_rest_args_positional {
-            arg_comp = ArgComp::CommandOrPositional;
-        }
-
-        let last_cmd = cmds.last().unwrap().1;
-        choices_fns.extend(last_cmd.positional_params.iter().filter_map(|v| {
-            if let Some((choices_fn, validate)) = v.choices_fn.as_ref() {
-                if *validate {
-                    return Some(choices_fn.as_str());
-                }
+            if is_rest_args_positional {
+                arg_comp = ArgComp::CommandOrPositional;
             }
-            None
-        }));
+
+            let last_cmd = cmds.last().unwrap().1;
+            choices_fns.extend(last_cmd.positional_params.iter().filter_map(|v| {
+                if let Some((choices_fn, validate)) = v.choices_fn.as_ref() {
+                    if *validate {
+                        return Some(choices_fn.as_str());
+                    }
+                }
+                None
+            }));
+        }
         Self {
             cmds,
             args,
@@ -184,7 +202,6 @@ impl<'a, 'b> Matcher<'a, 'b> {
             dashes,
             arg_comp,
             choices_fns,
-            choices_values: HashMap::new(),
             script_path: None,
             term_width: None,
             is_last_arg_option_assign,
@@ -193,26 +210,6 @@ impl<'a, 'b> Matcher<'a, 'b> {
 
     pub(crate) fn set_script_path(&mut self, script_path: &str) {
         self.script_path = Some(script_path.to_string());
-        let fns: Vec<&str> = self.choices_fns.iter().copied().collect();
-        let mut envs = HashMap::new();
-        envs.insert("ARGC_OS".into(), env::consts::OS.to_string());
-        envs.insert("ARGC_PATH_SEP".into(), MAIN_SEPARATOR.into());
-        if let Some(outputs) = run_param_fns(script_path, &fns, self.args, envs) {
-            for (i, output) in outputs.into_iter().enumerate() {
-                let choices = output
-                    .split('\n')
-                    .filter_map(|v| {
-                        let v = v.trim_matches(|c: char| c.is_whitespace() || c == '\0');
-                        if v.is_empty() {
-                            None
-                        } else {
-                            Some(v.to_string())
-                        }
-                    })
-                    .collect();
-                self.choices_values.insert(fns[i], choices);
-            }
-        }
     }
 
     pub(crate) fn set_term_width(&mut self, term_width: usize) {
@@ -247,9 +244,28 @@ impl<'a, 'b> Matcher<'a, 'b> {
         output
     }
 
-    pub(crate) fn compgen(&self) -> Vec<CompItem> {
+    pub(crate) fn compgen(&self, shell: Shell) -> Vec<CompItem> {
+        let redirect_symbols = shell.redirect_symbols();
+        if self
+            .args
+            .iter()
+            .any(|v| redirect_symbols.contains(&v.as_str()))
+        {
+            return vec![(
+                "__argc_value=path".into(),
+                String::new(),
+                CompColor::of_value(),
+            )];
+        }
         let level = self.cmds.len() - 1;
         let mut last_cmd = self.cmds[level].1;
+        if last_cmd.delegated() {
+            return last_cmd
+                .positional_params
+                .first()
+                .map(comp_positional)
+                .unwrap_or_default();
+        }
         let mut output = match &self.arg_comp {
             ArgComp::FlagOrOption => {
                 let mut output = self.comp_flag_options();
@@ -260,9 +276,9 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 {
                     let describe = param.describe_head();
                     let kind = if param.is_flag() {
-                        CompKind::Flag
+                        CompColor::of_flag()
                     } else {
-                        CompKind::Option
+                        CompColor::of_option()
                     };
                     output.push((value.clone(), describe.into(), kind));
                 }
@@ -281,7 +297,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                     })
                     .collect();
                 if output.len() == 1 {
-                    output.insert(0, (value.to_string(), String::new(), CompKind::Flag));
+                    output.insert(0, (value.to_string(), String::new(), CompColor::of_flag()));
                 }
                 output
             }
@@ -313,8 +329,15 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 comp_subcommands_positional(last_cmd, &values, self.positional_args.len() < 2)
             }
         };
-        if output.is_empty() && !self.arg_comp.is_flag_or_option() {
-            output.push(("__argc_value=file".into(), String::new(), CompKind::Value));
+        if output.is_empty()
+            && !self.arg_comp.is_flag_or_option()
+            && last_cmd.positional_params.is_empty()
+        {
+            output.push((
+                "__argc_value=path".into(),
+                String::new(),
+                CompColor::of_value(),
+            ));
         }
         output
     }
@@ -439,10 +462,13 @@ impl<'a, 'b> Matcher<'a, 'b> {
         } else {
             vec![]
         };
+
+        let choices_fn_values = self.run_choices_fns().unwrap_or_default();
+
         for (i, param) in last_cmd.positional_params.iter().enumerate() {
             if let (Some(values), Some(choices)) = (
                 positional_values.get(i),
-                get_param_choices(&param.choices, &param.choices_fn, &self.choices_values),
+                get_param_choices(&param.choices, &param.choices_fn, &choices_fn_values),
             ) {
                 for value in values.iter() {
                     if !choices.contains(&value.to_string()) {
@@ -504,11 +530,9 @@ impl<'a, 'b> Matcher<'a, 'b> {
                                 ));
                             }
                         }
-                        if let Some(choices) = get_param_choices(
-                            &param.choices,
-                            &param.choices_fn,
-                            &self.choices_values,
-                        ) {
+                        if let Some(choices) =
+                            get_param_choices(&param.choices, &param.choices_fn, &choices_fn_values)
+                        {
                             for value in values.iter() {
                                 if !choices.contains(&value.to_string()) {
                                     return Some(MatchError::InvalidValue(
@@ -537,6 +561,31 @@ impl<'a, 'b> Matcher<'a, 'b> {
         None
     }
 
+    fn run_choices_fns(&'a self) -> Option<HashMap<&'a str, Vec<String>>> {
+        let script_path = self.script_path.as_ref()?;
+        let mut choices_fn_values = HashMap::new();
+        let fns: Vec<&str> = self.choices_fns.iter().copied().collect();
+        let mut envs = HashMap::new();
+        envs.insert("ARGC_OS".into(), env::consts::OS.to_string());
+        envs.insert("ARGC_PATH_SEP".into(), MAIN_SEPARATOR.into());
+        let outputs = run_param_fns(script_path, &fns, self.args, envs)?;
+        for (i, output) in outputs.into_iter().enumerate() {
+            let choices = output
+                .split('\n')
+                .filter_map(|v| {
+                    let v = v.trim_matches(|c: char| c.is_whitespace() || c == '\0');
+                    if v.is_empty() {
+                        None
+                    } else {
+                        Some(v.to_string())
+                    }
+                })
+                .collect();
+            choices_fn_values.insert(fns[i], choices);
+        }
+        Some(choices_fn_values)
+    }
+
     fn match_positionals(&self) -> Vec<Vec<&str>> {
         let mut output = vec![];
         let args_len = self.positional_args.len();
@@ -719,9 +768,9 @@ impl<'a, 'b> Matcher<'a, 'b> {
             if !exist || param.multiple {
                 let describe = param.describe_head();
                 let kind = if param.is_flag() {
-                    CompKind::Flag
+                    CompColor::of_flag()
                 } else {
-                    CompKind::Option
+                    CompColor::of_option()
                 };
                 for v in param.list_names() {
                     output.push((v, describe.to_string(), kind))
@@ -732,7 +781,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
     }
 }
 
-pub(crate) type CompItem = (String, String, CompKind);
+pub(crate) type CompItem = (String, String, CompColor);
 
 fn add_positional_arg<'a>(
     positional_args: &mut Vec<&'a str>,
@@ -845,7 +894,7 @@ fn comp_subcomands(cmd: &Command) -> Vec<CompItem> {
     for subcmd in cmd.subcommands.iter() {
         let describe = subcmd.describe_head();
         for v in subcmd.list_names() {
-            output.push((v, describe.to_string(), CompKind::Command))
+            output.push((v, describe.to_string(), CompColor::of_command()))
         }
     }
     output
@@ -894,17 +943,17 @@ fn comp_param(
         match choices {
             Either::Left(choices) => choices
                 .iter()
-                .map(|v| (v.to_string(), String::new(), CompKind::Value))
+                .map(|v| (v.to_string(), String::new(), CompColor::of_value()))
                 .collect(),
             Either::Right(choices_fn) => vec![(
                 format!("__argc_fn={}", choices_fn),
                 String::new(),
-                CompKind::Value,
+                CompColor::of_value(),
             )],
         }
     } else {
         let value = format!("__argc_value={}", value_name);
-        vec![(value, describe.into(), CompKind::Value)]
+        vec![(value, describe.into(), CompColor::of_value())]
     };
     if let Some(ch) = multi_char {
         output.insert(
@@ -912,7 +961,7 @@ fn comp_param(
             (
                 format!("__argc_multi={}", ch),
                 String::new(),
-                CompKind::Value,
+                CompColor::of_value(),
             ),
         );
     }
@@ -922,12 +971,12 @@ fn comp_param(
 fn get_param_choices<'a, 'b: 'a>(
     choices: &'a Option<Vec<String>>,
     choices_fn: &'a Option<(String, bool)>,
-    choices_values: &'a HashMap<&str, Vec<String>>,
+    choices_fn_values: &'a HashMap<&str, Vec<String>>,
 ) -> Option<&'a Vec<String>> {
     choices.as_ref().or_else(|| {
         choices_fn.as_ref().and_then(|(fn_name, validate)| {
             if *validate {
-                choices_values.get(fn_name.as_str())
+                choices_fn_values.get(fn_name.as_str())
             } else {
                 None
             }
