@@ -2,6 +2,7 @@ use crate::param::{FlagOptionParam, ParamData, PositionalParam};
 use crate::utils::{is_choice_value_terminate, is_default_value_terminate};
 use crate::Result;
 use anyhow::bail;
+use nom::error::ErrorKind;
 use nom::{
     branch::alt,
     bytes::complete::{escaped, tag, take_till, take_while1},
@@ -490,7 +491,7 @@ fn parse_name(input: &str) -> nom::IResult<&str, &str> {
 }
 
 fn parse_multi_char(input: &str) -> nom::IResult<&str, char> {
-    one_of(",:;@|")(input)
+    one_of(",:;@|/")(input)
 }
 
 fn parse_default_value(input: &str) -> nom::IResult<&str, &str> {
@@ -524,7 +525,12 @@ fn parse_quoted_string(input: &str) -> nom::IResult<&str, &str> {
 
 fn parse_notation_text(input: &str) -> nom::IResult<&str, &str> {
     let (_, size) = notation_text(input, 1)?;
-    Ok((&input[size - 1..], &input[0..size - 1]))
+    let (offset, _) = input
+        .char_indices()
+        .nth(size - 1)
+        .ok_or_else(|| create_err(input, ErrorKind::Eof))?;
+    let (x, y) = input.split_at(offset);
+    Ok((y, x))
 }
 
 fn parse_normal_comment(input: &str) -> nom::IResult<&str, &str> {
@@ -567,14 +573,15 @@ fn verify_single_char(input: &str) -> nom::IResult<&str, &str> {
         .count()
         > 1
     {
-        return Err(nom::Err::Error(nom::error::Error::new(
-            input,
-            nom::error::ErrorKind::Verify,
-        )));
+        return Err(create_err(input, ErrorKind::Verify));
     }
     Ok((input, ""))
 }
 
+fn create_err(input: &str, kind: ErrorKind) -> nom::Err<nom::error::Error<&str>> {
+    nom::Err::Error(nom::error::Error::new(input, kind))
+}
+
 fn is_not_fn_name_char(c: char) -> bool {
     !matches!(
         c,
@@ -834,6 +841,11 @@ mod tests {
         assert_parse_positional_arg!("foo*[=a|b]");
     }
 
+    #[test]
+    fn test_special_notation() {
+        assert_parse_option_arg!("--foo <◉>");
+    }
+
     #[test]
     fn test_parse_line() {
         assert_token!("# @describe A demo cli", Describe, "A demo cli");
