@@ -404,6 +404,21 @@ _choice_fn() {
     snapshot_compgen_shells!(script, vec!["prog", "--oa", ""]);
 }
 
+#[test]
+fn suffix() {
+    let script = r###"
+# @option --oa*[`_choice_fn`]
+_choice_fn() {
+    echo -e "__argc_suffix==\0"
+	echo -e "abc"
+	echo -e "def"
+	echo -e "ghk"
+}
+"###;
+
+    snapshot_compgen_shells!(script, vec!["prog", "--oa", ""]);
+}
+
 #[test]
 fn value_display() {
     let script = r###"
@@ -457,7 +472,7 @@ fn assing_option_value() {
 # @arg val[`_choice_fn`]
 _choice_fn() {
     echo __argc_filter=
-    ( set -o posix ; set ) | grep 'argc_\|ARGC_' | grep -v 'ARGC_PWD\|ARGC_PATH_SEP\|ARGC_OS'
+    ( set -o posix ; set ) | grep 'argc_\|ARGC_COMPGEN\|ARGC_FILTER\|ARGC_LAST_ARG\|ARGC_VARS' 
 }
 "###;
 
@@ -565,6 +580,33 @@ cmd() {
     );
 }
 
+#[test]
+fn redirect_symbols() {
+    let script = r###"
+# @option --oa
+# @arg text*
+"###;
+
+    snapshot_compgen!(script, vec![vec!["prog", ">", "Argc"]], argc::Shell::Bash);
+}
+
+#[test]
+fn delegated() {
+    let script = r###"
+# @arg args~[`_choice_delegate`]
+
+_choice_delegate() {
+    echo $1
+}
+"###;
+
+    snapshot_compgen!(
+        script,
+        vec![vec!["prog", "abc"], vec!["prog", "-a"], vec!["prog", "-"]],
+        argc::Shell::Bash
+    );
+}
+
 #[test]
 fn mult_char() {
     let script = r###"
@@ -618,8 +660,11 @@ fn desc() {
     let script = r###"
 # @option --oa[`_choice_fn`]
 _choice_fn() {
-    echo -e "abc\t(desc1)"
-    echo -e "def\t(desc2)"
+    echo -e "abc\tdesc"
+    echo -e "def\t(desc) "
+    echo -e " ijk\t  value (desc)"
+    echo -e " xyz \t[desc]"
+    echo -e " cjk\t福聲幸雪弓們家扒乍植哪黑信，坡也士背文反四未間美穿八和經。何朵申別兆洋行苗青誰圓弓葉福音語：向哭扒長次友誰員完"
 }
 "###;
 
@@ -696,7 +741,7 @@ _choice_fn() {
     echo -e "c1\t"
     echo -e "d1\0"
     echo -e "e1"
-    echo -e "f1\t/kind:valueOther\tdesc f1"
+    echo -e "f1\t/color:yellow\tdesc f1"
 }
 "###;
 
@@ -719,33 +764,37 @@ _choice_fn() {
     snapshot_compgen_shells!(script, vec!["prog", "v='"]);
 }
 
-#[test]
-fn redirect_symbols() {
-    let script = r###"
-# @option --oa
-# @arg text*
-"###;
-
-    snapshot_compgen_shells!(script, vec!["prog", ">", ""]);
-}
-
 #[test]
 fn color() {
     let script = r###"
 # @arg val[`_fn_color`]
 _fn_color() {
-    echo -e "flag\0\t/kind:flag"
-    echo -e "option\0\t/kind:option"
-    echo -e "command\0\t/kind:command"
-    echo -e "dir\0\t/kind:dir"
-    echo -e "file\0\t/kind:file"
-    echo -e "fileExe\0\t/kind:fileExe"
-    echo -e "symlink\0\t/kind:symlink"
-    echo -e "valueOther\0\t/kind:valueOther"
-    echo -e "valueAnother\0\t/kind:valueAnother"
-    echo -e "valueEmphasis\0\t/kind:valueEmphasis"
-    echo -e "valueSubtle\0\t/kind:valueSubtle"
-    echo -e "value\0\t/kind:value"
+    echo -e "kindFlag\0\t/color:cyan"
+    echo -e "kindOption\0\t/color:cyan,bold"
+    echo -e "kindCommand\0\t/color:magenta"
+    echo -e "kindDir\0\t/color:blue,bold"
+    echo -e "kindFile\0\t/color:default"
+    echo -e "kindFileExe\0\t/color:green,bold"
+    echo -e "kindSymlink\0\t/color:cyan,bold"
+    echo -e "kindValue\0\t/color:green"
+    echo -e "colorBlack\0\t/color:black"
+    echo -e "colorBlackBold\0\t/color:black,bold"
+    echo -e "colorRed\0\t/color:red"
+    echo -e "colorRedBold\0\t/color:red,bold"
+    echo -e "colorGreen\0\t/color:green"
+    echo -e "colorGreenBold\0\t/color:green,bold"
+    echo -e "colorYellow\0\t/color:yellow"
+    echo -e "colorYellowBold\0\t/color:yellow,bold"
+    echo -e "colorBlue\0\t/color:blue"
+    echo -e "colorBlueBold\0\t/color:blue,bold"
+    echo -e "colorMagenta\0\t/color:magenta"
+    echo -e "colorMagentaBold\0\t/color:magenta,bold"
+    echo -e "colorCyan\0\t/color:cyan"
+    echo -e "colorCyanBold\0\t/color:cyan,bold"
+    echo -e "colorWhite\0\t/color:white"
+    echo -e "colorWhiteBold\0\t/color:white,bold"
+    echo -e "colorDefault\0\t/color:default"
+    echo -e "colorDefaultBold\0\t/color:default,bold"
 }
 "###;
     snapshot_compgen_shells!(script, vec!["prog", ""]);
@@ -800,13 +849,19 @@ mod filedir {
     }
 
     const CD_SCRIPT: &str = r###"
-# @option --oa[`_choice_fn`]
-# @arg val[`_choice_fn2`]
-_choice_fn() {
+# @option --oa[`_choice_oa`]
+# @option --ob[`_choice_ob`]
+# @arg val[`choice_val`]
+_choice_oa() {
     echo "__argc_cd=src"
     echo "__argc_value=file"
 }
-_choice_fn2() {
+_choice_ob() {
+    echo -e "__argc_suffix=:\0"
+    echo "__argc_cd=src"
+    echo "__argc_value=file"
+}
+choice_val() {
     if [[ "$1" == *"="* ]]; then
         echo __argc_prefix=${1%%=*}=
         echo __argc_filter=${1#*=}
@@ -825,6 +880,7 @@ _choice_fn2() {
             vec![
                 vec!["prog", "--oa", ""],
                 vec!["prog", "--oa="],
+                vec!["prog", "--ob", ""],
                 vec!["prog", "foo="]
             ],
             TEST_SHELL
@@ -838,6 +894,7 @@ _choice_fn2() {
             CD_SCRIPT,
             vec![
                 vec!["prog", "--oa", ""],
+                vec!["prog", "--ob", ""],
                 vec!["prog", "--oa="],
                 vec!["prog", "foo="]
             ],
